@@ -12,15 +12,15 @@ fi
 laf="$GIT_DIR/lost-found"
 rm -fr "$laf" && mkdir -p "$laf/commit" "$laf/other" || exit
 
-git fsck-objects |
+git fsck --full --no-reflogs |
 while read dangling type sha1
 do
 	case "$dangling" in
 	dangling)
-		if git-rev-parse --verify "$sha1^0" >/dev/null 2>/dev/null
+		if git rev-parse --verify "$sha1^0" >/dev/null 2>/dev/null
 		then
 			dir="$laf/commit"
-			git-show-branch "$sha1"
+			git show-branch "$sha1"
 		else
 			dir="$laf/other"
 		fi
