@@ -3,8 +3,11 @@
  *
  * Copyright (C) Linus Torvalds, 2005
  */
+#define NO_THE_INDEX_COMPATIBILITY_MACROS
 #include "cache.h"
 #include "cache-tree.h"
+#include "refs.h"
+#include "dir.h"
 
 /* Index extensions.
  *
@@ -18,16 +21,7 @@
 #define CACHE_EXT(s) ( (s[0]<<24)|(s[1]<<16)|(s[2]<<8)|(s[3]) )
 #define CACHE_EXT_TREE 0x54524545	/* "TREE" */
 
-struct cache_entry **active_cache;
-static time_t index_file_timestamp;
-unsigned int active_nr, active_alloc, active_cache_changed;
-
-struct cache_tree *active_cache_tree;
-
-int cache_errno;
-
-static void *cache_mmap;
-static size_t cache_mmap_size;
+struct index_state the_index;
 
 /*
  * This only updates the "non-critical" parts of the directory
@@ -59,20 +53,20 @@ static int ce_compare_data(struct cache_entry *ce, struct stat *st)
 
 	if (fd >= 0) {
 		unsigned char sha1[20];
-		if (!index_fd(sha1, fd, st, 0, NULL))
+		if (!index_fd(sha1, fd, st, 0, OBJ_BLOB, ce->name))
 			match = hashcmp(sha1, ce->sha1);
 		/* index_fd() closed the file descriptor already */
 	}
 	return match;
 }
 
-static int ce_compare_link(struct cache_entry *ce, unsigned long expected_size)
+static int ce_compare_link(struct cache_entry *ce, size_t expected_size)
 {
 	int match = -1;
 	char *target;
 	void *buffer;
 	unsigned long size;
-	char type[10];
+	enum object_type type;
 	int len;
 
 	target = xmalloc(expected_size);
@@ -81,7 +75,7 @@ static int ce_compare_link(struct cache_entry *ce, unsigned long expected_size)
 		free(target);
 		return -1;
 	}
-	buffer = read_sha1_file(ce->sha1, type, &size);
+	buffer = read_sha1_file(ce->sha1, &type, &size);
 	if (!buffer) {
 		free(target);
 		return -1;
@@ -93,6 +87,23 @@ static int ce_compare_link(struct cache_entry *ce, unsigned long expected_size)
 	return match;
 }
 
+static int ce_compare_gitlink(struct cache_entry *ce)
+{
+	unsigned char sha1[20];
+
+	/*
+	 * We don't actually require that the .git directory
+	 * under GITLINK directory be a valid git directory. It
+	 * might even be missing (in case nobody populated that
+	 * sub-project).
+	 *
+	 * If so, we consider it always to match.
+	 */
+	if (resolve_gitlink_ref(ce->name, "HEAD", sha1) < 0)
+		return 0;
+	return hashcmp(sha1, ce->sha1);
+}
+
 static int ce_modified_check_fs(struct cache_entry *ce, struct stat *st)
 {
 	switch (st->st_mode & S_IFMT) {
@@ -101,9 +112,12 @@ static int ce_modified_check_fs(struct cache_entry *ce, struct stat *st)
 			return DATA_CHANGED;
 		break;
 	case S_IFLNK:
-		if (ce_compare_link(ce, st->st_size))
+		if (ce_compare_link(ce, xsize_t(st->st_size)))
 			return DATA_CHANGED;
 		break;
+	case S_IFDIR:
+		if (S_ISGITLINK(ntohl(ce->ce_mode)))
+			return 0;
 	default:
 		return TYPE_CHANGED;
 	}
@@ -125,8 +139,18 @@ static int ce_match_stat_basic(struct cache_entry *ce, struct stat *st)
 			changed |= MODE_CHANGED;
 		break;
 	case S_IFLNK:
-		changed |= !S_ISLNK(st->st_mode) ? TYPE_CHANGED : 0;
+		if (!S_ISLNK(st->st_mode) &&
+		    (has_symlinks || !S_ISREG(st->st_mode)))
+			changed |= TYPE_CHANGED;
 		break;
+	case S_IFGITLINK:
+		if (!S_ISDIR(st->st_mode))
+			changed |= TYPE_CHANGED;
+		else if (ce_compare_gitlink(ce))
+			changed |= DATA_CHANGED;
+		return changed;
+	case 0: /* Special case: unmerged file in index */
+		return MODE_CHANGED | DATA_CHANGED | TYPE_CHANGED;
 	default:
 		die("internal error: ce_mode is %o", ntohl(ce->ce_mode));
 	}
@@ -145,7 +169,7 @@ static int ce_match_stat_basic(struct cache_entry *ce, struct stat *st)
 		changed |= MTIME_CHANGED;
 	if (ce->ce_ctime.nsec != htonl(st->st_ctim.tv_nsec))
 		changed |= CTIME_CHANGED;
-#endif	
+#endif
 
 	if (ce->ce_uid != htonl(st->st_uid) ||
 	    ce->ce_gid != htonl(st->st_gid))
@@ -169,7 +193,8 @@ static int ce_match_stat_basic(struct cache_entry *ce, struct stat *st)
 	return changed;
 }
 
-int ce_match_stat(struct cache_entry *ce, struct stat *st, int options)
+int ie_match_stat(struct index_state *istate,
+		  struct cache_entry *ce, struct stat *st, int options)
 {
 	unsigned int changed;
 	int ignore_valid = options & 01;
@@ -201,8 +226,8 @@ int ce_match_stat(struct cache_entry *ce, struct stat *st, int options)
 	 * carefully than others.
 	 */
 	if (!changed &&
-	    index_file_timestamp &&
-	    index_file_timestamp <= ntohl(ce->ce_mtime.sec)) {
+	    istate->timestamp &&
+	    istate->timestamp <= ntohl(ce->ce_mtime.sec)) {
 		if (assume_racy_is_modified)
 			changed |= DATA_CHANGED;
 		else
@@ -212,10 +237,11 @@ int ce_match_stat(struct cache_entry *ce, struct stat *st, int options)
 	return changed;
 }
 
-int ce_modified(struct cache_entry *ce, struct stat *st, int really)
+int ie_modified(struct index_state *istate,
+		struct cache_entry *ce, struct stat *st, int really)
 {
 	int changed, changed_fs;
-	changed = ce_match_stat(ce, st, really);
+	changed = ie_match_stat(istate, ce, st, really);
 	if (!changed)
 		return 0;
 	/*
@@ -283,15 +309,15 @@ int cache_name_compare(const char *name1, int flags1, const char *name2, int fla
 	return 0;
 }
 
-int cache_name_pos(const char *name, int namelen)
+int index_name_pos(struct index_state *istate, const char *name, int namelen)
 {
 	int first, last;
 
 	first = 0;
-	last = active_nr;
+	last = istate->cache_nr;
 	while (last > first) {
 		int next = (last + first) >> 1;
-		struct cache_entry *ce = active_cache[next];
+		struct cache_entry *ce = istate->cache[next];
 		int cmp = cache_name_compare(name, namelen, ce->name, ntohs(ce->ce_flags));
 		if (!cmp)
 			return next;
@@ -305,67 +331,136 @@ int cache_name_pos(const char *name, int namelen)
 }
 
 /* Remove entry, return true if there are more entries to go.. */
-int remove_cache_entry_at(int pos)
+int remove_index_entry_at(struct index_state *istate, int pos)
 {
-	active_cache_changed = 1;
-	active_nr--;
-	if (pos >= active_nr)
+	istate->cache_changed = 1;
+	istate->cache_nr--;
+	if (pos >= istate->cache_nr)
 		return 0;
-	memmove(active_cache + pos, active_cache + pos + 1, (active_nr - pos) * sizeof(struct cache_entry *));
+	memmove(istate->cache + pos,
+		istate->cache + pos + 1,
+		(istate->cache_nr - pos) * sizeof(struct cache_entry *));
 	return 1;
 }
 
-int remove_file_from_cache(const char *path)
+int remove_file_from_index(struct index_state *istate, const char *path)
 {
-	int pos = cache_name_pos(path, strlen(path));
+	int pos = index_name_pos(istate, path, strlen(path));
 	if (pos < 0)
 		pos = -pos-1;
-	while (pos < active_nr && !strcmp(active_cache[pos]->name, path))
-		remove_cache_entry_at(pos);
+	while (pos < istate->cache_nr && !strcmp(istate->cache[pos]->name, path))
+		remove_index_entry_at(istate, pos);
 	return 0;
 }
 
-int add_file_to_index(const char *path, int verbose)
+static int compare_name(struct cache_entry *ce, const char *path, int namelen)
 {
-	int size, namelen;
+	return namelen != ce_namelen(ce) || memcmp(path, ce->name, namelen);
+}
+
+static int index_name_pos_also_unmerged(struct index_state *istate,
+	const char *path, int namelen)
+{
+	int pos = index_name_pos(istate, path, namelen);
+	struct cache_entry *ce;
+
+	if (pos >= 0)
+		return pos;
+
+	/* maybe unmerged? */
+	pos = -1 - pos;
+	if (pos >= istate->cache_nr ||
+			compare_name((ce = istate->cache[pos]), path, namelen))
+		return -1;
+
+	/* order of preference: stage 2, 1, 3 */
+	if (ce_stage(ce) == 1 && pos + 1 < istate->cache_nr &&
+			ce_stage((ce = istate->cache[pos + 1])) == 2 &&
+			!compare_name(ce, path, namelen))
+		pos++;
+	return pos;
+}
+
+int add_file_to_index(struct index_state *istate, const char *path, int verbose)
+{
+	int size, namelen, pos;
 	struct stat st;
 	struct cache_entry *ce;
 
 	if (lstat(path, &st))
 		die("%s: unable to stat (%s)", path, strerror(errno));
 
-	if (!S_ISREG(st.st_mode) && !S_ISLNK(st.st_mode))
-		die("%s: can only add regular files or symbolic links", path);
+	if (!S_ISREG(st.st_mode) && !S_ISLNK(st.st_mode) && !S_ISDIR(st.st_mode))
+		die("%s: can only add regular files, symbolic links or git-directories", path);
 
 	namelen = strlen(path);
+	if (S_ISDIR(st.st_mode)) {
+		while (namelen && path[namelen-1] == '/')
+			namelen--;
+	}
 	size = cache_entry_size(namelen);
 	ce = xcalloc(1, size);
 	memcpy(ce->name, path, namelen);
 	ce->ce_flags = htons(namelen);
 	fill_stat_cache_info(ce, &st);
 
-	ce->ce_mode = create_ce_mode(st.st_mode);
-	if (!trust_executable_bit) {
-		/* If there is an existing entry, pick the mode bits
-		 * from it, otherwise assume unexecutable.
+	if (trust_executable_bit && has_symlinks)
+		ce->ce_mode = create_ce_mode(st.st_mode);
+	else {
+		/* If there is an existing entry, pick the mode bits and type
+		 * from it, otherwise assume unexecutable regular file.
 		 */
-		int pos = cache_name_pos(path, namelen);
-		if (pos >= 0)
-			ce->ce_mode = active_cache[pos]->ce_mode;
-		else if (S_ISREG(st.st_mode))
-			ce->ce_mode = create_ce_mode(S_IFREG | 0666);
+		struct cache_entry *ent;
+		int pos = index_name_pos_also_unmerged(istate, path, namelen);
+
+		ent = (0 <= pos) ? istate->cache[pos] : NULL;
+		ce->ce_mode = ce_mode_from_stat(ent, st.st_mode);
+	}
+
+	pos = index_name_pos(istate, ce->name, namelen);
+	if (0 <= pos &&
+	    !ce_stage(istate->cache[pos]) &&
+	    !ie_modified(istate, istate->cache[pos], &st, 1)) {
+		/* Nothing changed, really */
+		free(ce);
+		return 0;
 	}
 
 	if (index_path(ce->sha1, path, &st, 1))
 		die("unable to index file %s", path);
-	if (add_cache_entry(ce, ADD_CACHE_OK_TO_ADD))
+	if (add_index_entry(istate, ce, ADD_CACHE_OK_TO_ADD|ADD_CACHE_OK_TO_REPLACE))
 		die("unable to add %s to index",path);
 	if (verbose)
 		printf("add '%s'\n", path);
-	cache_tree_invalidate_path(active_cache_tree, path);
+	cache_tree_invalidate_path(istate->cache_tree, path);
 	return 0;
 }
 
+struct cache_entry *make_cache_entry(unsigned int mode,
+		const unsigned char *sha1, const char *path, int stage,
+		int refresh)
+{
+	int size, len;
+	struct cache_entry *ce;
+
+	if (!verify_path(path))
+		return NULL;
+
+	len = strlen(path);
+	size = cache_entry_size(len);
+	ce = xcalloc(1, size);
+
+	hashcpy(ce->sha1, sha1);
+	memcpy(ce->name, path, len);
+	ce->ce_flags = create_ce_flags(len, stage);
+	ce->ce_mode = create_ce_mode(mode);
+
+	if (refresh)
+		return refresh_cache_entry(ce, 0);
+
+	return ce;
+}
+
 int ce_same_name(struct cache_entry *a, struct cache_entry *b)
 {
 	int len = ce_namelen(a);
@@ -466,15 +561,16 @@ inside:
  * Do we have another file that has the beginning components being a
  * proper superset of the name we're trying to add?
  */
-static int has_file_name(const struct cache_entry *ce, int pos, int ok_to_replace)
+static int has_file_name(struct index_state *istate,
+			 const struct cache_entry *ce, int pos, int ok_to_replace)
 {
 	int retval = 0;
 	int len = ce_namelen(ce);
 	int stage = ce_stage(ce);
 	const char *name = ce->name;
 
-	while (pos < active_nr) {
-		struct cache_entry *p = active_cache[pos++];
+	while (pos < istate->cache_nr) {
+		struct cache_entry *p = istate->cache[pos++];
 
 		if (len >= ce_namelen(p))
 			break;
@@ -484,10 +580,12 @@ static int has_file_name(const struct cache_entry *ce, int pos, int ok_to_replac
 			continue;
 		if (p->name[len] != '/')
 			continue;
+		if (!ce_stage(p) && !p->ce_mode)
+			continue;
 		retval = -1;
 		if (!ok_to_replace)
 			break;
-		remove_cache_entry_at(--pos);
+		remove_index_entry_at(istate, --pos);
 	}
 	return retval;
 }
@@ -496,7 +594,8 @@ static int has_file_name(const struct cache_entry *ce, int pos, int ok_to_replac
  * Do we have another file with a pathname that is a proper
  * subset of the name we're trying to add?
  */
-static int has_dir_name(const struct cache_entry *ce, int pos, int ok_to_replace)
+static int has_dir_name(struct index_state *istate,
+			const struct cache_entry *ce, int pos, int ok_to_replace)
 {
 	int retval = 0;
 	int stage = ce_stage(ce);
@@ -514,28 +613,39 @@ static int has_dir_name(const struct cache_entry *ce, int pos, int ok_to_replace
 		}
 		len = slash - name;
 
-		pos = cache_name_pos(name, ntohs(create_ce_flags(len, stage)));
+		pos = index_name_pos(istate, name, ntohs(create_ce_flags(len, stage)));
 		if (pos >= 0) {
-			retval = -1;
-			if (ok_to_replace)
-				break;
-			remove_cache_entry_at(pos);
-			continue;
+			/*
+			 * Found one, but not so fast.  This could
+			 * be a marker that says "I was here, but
+			 * I am being removed".  Such an entry is
+			 * not a part of the resulting tree, and
+			 * it is Ok to have a directory at the same
+			 * path.
+			 */
+			if (stage || istate->cache[pos]->ce_mode) {
+				retval = -1;
+				if (!ok_to_replace)
+					break;
+				remove_index_entry_at(istate, pos);
+				continue;
+			}
 		}
+		else
+			pos = -pos-1;
 
 		/*
 		 * Trivial optimization: if we find an entry that
 		 * already matches the sub-directory, then we know
 		 * we're ok, and we can exit.
 		 */
-		pos = -pos-1;
-		while (pos < active_nr) {
-			struct cache_entry *p = active_cache[pos];
+		while (pos < istate->cache_nr) {
+			struct cache_entry *p = istate->cache[pos];
 			if ((ce_namelen(p) <= len) ||
 			    (p->name[len] != '/') ||
 			    memcmp(p->name, name, len))
 				break; /* not our subdirectory */
-			if (ce_stage(p) == stage)
+			if (ce_stage(p) == stage && (stage || p->ce_mode))
 				/* p is at the same stage as our entry, and
 				 * is a subdirectory of what we are looking
 				 * at, so we cannot have conflicts at our
@@ -552,39 +662,50 @@ static int has_dir_name(const struct cache_entry *ce, int pos, int ok_to_replace
  * is being added, or we already have path and path/file is being
  * added.  Either one would result in a nonsense tree that has path
  * twice when git-write-tree tries to write it out.  Prevent it.
- * 
+ *
  * If ok-to-replace is specified, we remove the conflicting entries
  * from the cache so the caller should recompute the insert position.
  * When this happens, we return non-zero.
  */
-static int check_file_directory_conflict(const struct cache_entry *ce, int pos, int ok_to_replace)
+static int check_file_directory_conflict(struct index_state *istate,
+					 const struct cache_entry *ce,
+					 int pos, int ok_to_replace)
 {
+	int retval;
+
+	/*
+	 * When ce is an "I am going away" entry, we allow it to be added
+	 */
+	if (!ce_stage(ce) && !ce->ce_mode)
+		return 0;
+
 	/*
 	 * We check if the path is a sub-path of a subsequent pathname
 	 * first, since removing those will not change the position
-	 * in the array
+	 * in the array.
 	 */
-	int retval = has_file_name(ce, pos, ok_to_replace);
+	retval = has_file_name(istate, ce, pos, ok_to_replace);
+
 	/*
 	 * Then check if the path might have a clashing sub-directory
 	 * before it.
 	 */
-	return retval + has_dir_name(ce, pos, ok_to_replace);
+	return retval + has_dir_name(istate, ce, pos, ok_to_replace);
 }
 
-int add_cache_entry(struct cache_entry *ce, int option)
+static int add_index_entry_with_check(struct index_state *istate, struct cache_entry *ce, int option)
 {
 	int pos;
 	int ok_to_add = option & ADD_CACHE_OK_TO_ADD;
 	int ok_to_replace = option & ADD_CACHE_OK_TO_REPLACE;
 	int skip_df_check = option & ADD_CACHE_SKIP_DFCHECK;
 
-	pos = cache_name_pos(ce->name, ntohs(ce->ce_flags));
+	pos = index_name_pos(istate, ce->name, ntohs(ce->ce_flags));
 
 	/* existing match? Just replace it. */
 	if (pos >= 0) {
-		active_cache_changed = 1;
-		active_cache[pos] = ce;
+		istate->cache_changed = 1;
+		istate->cache[pos] = ce;
 		return 0;
 	}
 	pos = -pos-1;
@@ -593,10 +714,10 @@ int add_cache_entry(struct cache_entry *ce, int option)
 	 * Inserting a merged entry ("stage 0") into the index
 	 * will always replace all non-merged entries..
 	 */
-	if (pos < active_nr && ce_stage(ce) == 0) {
-		while (ce_same_name(active_cache[pos], ce)) {
+	if (pos < istate->cache_nr && ce_stage(ce) == 0) {
+		while (ce_same_name(istate->cache[pos], ce)) {
 			ok_to_add = 1;
-			if (!remove_cache_entry_at(pos))
+			if (!remove_index_entry_at(istate, pos))
 				break;
 		}
 	}
@@ -607,25 +728,45 @@ int add_cache_entry(struct cache_entry *ce, int option)
 		return -1;
 
 	if (!skip_df_check &&
-	    check_file_directory_conflict(ce, pos, ok_to_replace)) {
+	    check_file_directory_conflict(istate, ce, pos, ok_to_replace)) {
 		if (!ok_to_replace)
-			return -1;
-		pos = cache_name_pos(ce->name, ntohs(ce->ce_flags));
+			return error("'%s' appears as both a file and as a directory",
+				     ce->name);
+		pos = index_name_pos(istate, ce->name, ntohs(ce->ce_flags));
 		pos = -pos-1;
 	}
+	return pos + 1;
+}
+
+int add_index_entry(struct index_state *istate, struct cache_entry *ce, int option)
+{
+	int pos;
+
+	if (option & ADD_CACHE_JUST_APPEND)
+		pos = istate->cache_nr;
+	else {
+		int ret;
+		ret = add_index_entry_with_check(istate, ce, option);
+		if (ret <= 0)
+			return ret;
+		pos = ret - 1;
+	}
 
 	/* Make sure the array is big enough .. */
-	if (active_nr == active_alloc) {
-		active_alloc = alloc_nr(active_alloc);
-		active_cache = xrealloc(active_cache, active_alloc * sizeof(struct cache_entry *));
+	if (istate->cache_nr == istate->cache_alloc) {
+		istate->cache_alloc = alloc_nr(istate->cache_alloc);
+		istate->cache = xrealloc(istate->cache,
+					istate->cache_alloc * sizeof(struct cache_entry *));
 	}
 
 	/* Add it in.. */
-	active_nr++;
-	if (active_nr > pos)
-		memmove(active_cache + pos + 1, active_cache + pos, (active_nr - pos - 1) * sizeof(ce));
-	active_cache[pos] = ce;
-	active_cache_changed = 1;
+	istate->cache_nr++;
+	if (istate->cache_nr > pos + 1)
+		memmove(istate->cache + pos + 1,
+			istate->cache + pos,
+			(istate->cache_nr - pos - 1) * sizeof(ce));
+	istate->cache[pos] = ce;
+	istate->cache_changed = 1;
 	return 0;
 }
 
@@ -640,18 +781,20 @@ int add_cache_entry(struct cache_entry *ce, int option)
  * For example, you'd want to do this after doing a "git-read-tree",
  * to link up the stat cache details with the proper files.
  */
-struct cache_entry *refresh_cache_entry(struct cache_entry *ce, int really)
+static struct cache_entry *refresh_cache_ent(struct index_state *istate,
+					     struct cache_entry *ce, int really, int *err)
 {
 	struct stat st;
 	struct cache_entry *updated;
 	int changed, size;
 
 	if (lstat(ce->name, &st) < 0) {
-		cache_errno = errno;
+		if (err)
+			*err = errno;
 		return NULL;
 	}
 
-	changed = ce_match_stat(ce, &st, really);
+	changed = ie_match_stat(istate, ce, &st, really);
 	if (!changed) {
 		if (really && assume_unchanged &&
 		    !(ce->ce_flags & htons(CE_VALID)))
@@ -660,8 +803,9 @@ struct cache_entry *refresh_cache_entry(struct cache_entry *ce, int really)
 			return ce;
 	}
 
-	if (ce_modified(ce, &st, really)) {
-		cache_errno = EINVAL;
+	if (ie_modified(istate, ce, &st, really)) {
+		if (err)
+			*err = EINVAL;
 		return NULL;
 	}
 
@@ -682,7 +826,7 @@ struct cache_entry *refresh_cache_entry(struct cache_entry *ce, int really)
 	return updated;
 }
 
-int refresh_cache(unsigned int flags)
+int refresh_index(struct index_state *istate, unsigned int flags, const char **pathspec, char *seen)
 {
 	int i;
 	int has_errors = 0;
@@ -691,12 +835,14 @@ int refresh_cache(unsigned int flags)
 	int quiet = (flags & REFRESH_QUIET) != 0;
 	int not_new = (flags & REFRESH_IGNORE_MISSING) != 0;
 
-	for (i = 0; i < active_nr; i++) {
+	for (i = 0; i < istate->cache_nr; i++) {
 		struct cache_entry *ce, *new;
-		ce = active_cache[i];
+		int cache_errno = 0;
+
+		ce = istate->cache[i];
 		if (ce_stage(ce)) {
-			while ((i < active_nr) &&
-			       ! strcmp(active_cache[i]->name, ce->name))
+			while ((i < istate->cache_nr) &&
+			       ! strcmp(istate->cache[i]->name, ce->name))
 				i++;
 			i--;
 			if (allow_unmerged)
@@ -706,7 +852,10 @@ int refresh_cache(unsigned int flags)
 			continue;
 		}
 
-		new = refresh_cache_entry(ce, really);
+		if (pathspec && !match_pathspec(pathspec, ce->name, strlen(ce->name), 0, seen))
+			continue;
+
+		new = refresh_cache_ent(istate, ce, really, &cache_errno);
 		if (new == ce)
 			continue;
 		if (!new) {
@@ -717,7 +866,7 @@ int refresh_cache(unsigned int flags)
 				 * means the index is not valid anymore.
 				 */
 				ce->ce_flags &= ~htons(CE_VALID);
-				active_cache_changed = 1;
+				istate->cache_changed = 1;
 			}
 			if (quiet)
 				continue;
@@ -725,15 +874,20 @@ int refresh_cache(unsigned int flags)
 			has_errors = 1;
 			continue;
 		}
-		active_cache_changed = 1;
-		/* You can NOT just free active_cache[i] here, since it
+		istate->cache_changed = 1;
+		/* You can NOT just free istate->cache[i] here, since it
 		 * might not be necessarily malloc()ed but can also come
 		 * from mmap(). */
-		active_cache[i] = new;
+		istate->cache[i] = new;
 	}
 	return has_errors;
 }
 
+struct cache_entry *refresh_cache_entry(struct cache_entry *ce, int really)
+{
+	return refresh_cache_ent(&the_index, ce, really, NULL);
+}
+
 static int verify_hdr(struct cache_header *hdr, unsigned long size)
 {
 	SHA_CTX c;
@@ -751,11 +905,12 @@ static int verify_hdr(struct cache_header *hdr, unsigned long size)
 	return 0;
 }
 
-static int read_index_extension(const char *ext, void *data, unsigned long sz)
+static int read_index_extension(struct index_state *istate,
+				const char *ext, void *data, unsigned long sz)
 {
 	switch (CACHE_EXT(ext)) {
 	case CACHE_EXT_TREE:
-		active_cache_tree = cache_tree_read(data, sz);
+		istate->cache_tree = cache_tree_read(data, sz);
 		break;
 	default:
 		if (*ext < 'A' || 'Z' < *ext)
@@ -767,13 +922,13 @@ static int read_index_extension(const char *ext, void *data, unsigned long sz)
 	return 0;
 }
 
-int read_cache(void)
+int read_index(struct index_state *istate)
 {
-	return read_cache_from(get_index_file());
+	return read_index_from(istate, get_index_file());
 }
 
 /* remember to discard_cache() before reading a different cache! */
-int read_cache_from(const char *path)
+int read_index_from(struct index_state *istate, const char *path)
 {
 	int fd, i;
 	struct stat st;
@@ -781,11 +936,11 @@ int read_cache_from(const char *path)
 	struct cache_header *hdr;
 
 	errno = EBUSY;
-	if (cache_mmap)
-		return active_nr;
+	if (istate->mmap)
+		return istate->cache_nr;
 
 	errno = ENOENT;
-	index_file_timestamp = 0;
+	istate->timestamp = 0;
 	fd = open(path, O_RDONLY);
 	if (fd < 0) {
 		if (errno == ENOENT)
@@ -793,33 +948,35 @@ int read_cache_from(const char *path)
 		die("index file open failed (%s)", strerror(errno));
 	}
 
-	cache_mmap = MAP_FAILED;
-	if (!fstat(fd, &st)) {
-		cache_mmap_size = st.st_size;
-		errno = EINVAL;
-		if (cache_mmap_size >= sizeof(struct cache_header) + 20)
-			cache_mmap = mmap(NULL, cache_mmap_size, PROT_READ | PROT_WRITE, MAP_PRIVATE, fd, 0);
-	}
-	close(fd);
-	if (cache_mmap == MAP_FAILED)
-		die("index file mmap failed (%s)", strerror(errno));
+	if (fstat(fd, &st))
+		die("cannot stat the open index (%s)", strerror(errno));
 
-	hdr = cache_mmap;
-	if (verify_hdr(hdr, cache_mmap_size) < 0)
+	errno = EINVAL;
+	istate->mmap_size = xsize_t(st.st_size);
+	if (istate->mmap_size < sizeof(struct cache_header) + 20)
+		die("index file smaller than expected");
+
+	istate->mmap = xmmap(NULL, istate->mmap_size, PROT_READ | PROT_WRITE, MAP_PRIVATE, fd, 0);
+	close(fd);
+
+	hdr = istate->mmap;
+	if (verify_hdr(hdr, istate->mmap_size) < 0)
 		goto unmap;
 
-	active_nr = ntohl(hdr->hdr_entries);
-	active_alloc = alloc_nr(active_nr);
-	active_cache = xcalloc(active_alloc, sizeof(struct cache_entry *));
+	istate->cache_nr = ntohl(hdr->hdr_entries);
+	istate->cache_alloc = alloc_nr(istate->cache_nr);
+	istate->cache = xcalloc(istate->cache_alloc, sizeof(struct cache_entry *));
 
 	offset = sizeof(*hdr);
-	for (i = 0; i < active_nr; i++) {
-		struct cache_entry *ce = (struct cache_entry *) ((char *) cache_mmap + offset);
+	for (i = 0; i < istate->cache_nr; i++) {
+		struct cache_entry *ce;
+
+		ce = (struct cache_entry *)((char *)(istate->mmap) + offset);
 		offset = offset + ce_size(ce);
-		active_cache[i] = ce;
+		istate->cache[i] = ce;
 	}
-	index_file_timestamp = st.st_mtime;
-	while (offset <= cache_mmap_size - 20 - 8) {
+	istate->timestamp = st.st_mtime;
+	while (offset <= istate->mmap_size - 20 - 8) {
 		/* After an array of active_nr index entries,
 		 * there can be arbitrary number of extended
 		 * sections, each of which is prefixed with
@@ -827,35 +984,37 @@ int read_cache_from(const char *path)
 		 * in 4-byte network byte order.
 		 */
 		unsigned long extsize;
-		memcpy(&extsize, (char *) cache_mmap + offset + 4, 4);
+		memcpy(&extsize, (char *)(istate->mmap) + offset + 4, 4);
 		extsize = ntohl(extsize);
-		if (read_index_extension(((const char *) cache_mmap) + offset,
-					 (char *) cache_mmap + offset + 8,
+		if (read_index_extension(istate,
+					 ((const char *) (istate->mmap)) + offset,
+					 (char *) (istate->mmap) + offset + 8,
 					 extsize) < 0)
 			goto unmap;
 		offset += 8;
 		offset += extsize;
 	}
-	return active_nr;
+	return istate->cache_nr;
 
 unmap:
-	munmap(cache_mmap, cache_mmap_size);
+	munmap(istate->mmap, istate->mmap_size);
 	errno = EINVAL;
 	die("index file corrupt");
 }
 
-int discard_cache(void)
+int discard_index(struct index_state *istate)
 {
 	int ret;
 
-	active_nr = active_cache_changed = 0;
-	index_file_timestamp = 0;
-	cache_tree_free(&active_cache_tree);
-	if (cache_mmap == NULL)
+	istate->cache_nr = 0;
+	istate->cache_changed = 0;
+	istate->timestamp = 0;
+	cache_tree_free(&(istate->cache_tree));
+	if (istate->mmap == NULL)
 		return 0;
-	ret = munmap(cache_mmap, cache_mmap_size);
-	cache_mmap = NULL;
-	cache_mmap_size = 0;
+	ret = munmap(istate->mmap, istate->mmap_size);
+	istate->mmap = NULL;
+	istate->mmap_size = 0;
 
 	/* no need to throw away allocated active_cache */
 	return ret;
@@ -870,7 +1029,7 @@ static int ce_write_flush(SHA_CTX *context, int fd)
 	unsigned int buffered = write_buffer_len;
 	if (buffered) {
 		SHA1_Update(context, write_buffer, buffered);
-		if (write(fd, write_buffer, buffered) != buffered)
+		if (write_in_full(fd, write_buffer, buffered) != buffered)
 			return -1;
 		write_buffer_len = 0;
 	}
@@ -895,8 +1054,8 @@ static int ce_write(SHA_CTX *context, int fd, void *data, unsigned int len)
 		write_buffer_len = buffered;
 		len -= partial;
 		data = (char *) data + partial;
- 	}
- 	return 0;
+	}
+	return 0;
 }
 
 static int write_index_ext_header(SHA_CTX *context, int fd,
@@ -919,7 +1078,7 @@ static int ce_flush(SHA_CTX *context, int fd)
 
 	/* Flush first if not enough space for SHA1 signature */
 	if (left + 20 > WRITE_BUFFER_SIZE) {
-		if (write(fd, write_buffer, left) != left)
+		if (write_in_full(fd, write_buffer, left) != left)
 			return -1;
 		left = 0;
 	}
@@ -927,7 +1086,7 @@ static int ce_flush(SHA_CTX *context, int fd)
 	/* Append the SHA1 signature at the end */
 	SHA1_Final(write_buffer + left, context);
 	left += 20;
-	return (write(fd, write_buffer, left) != left) ? -1 : 0;
+	return (write_in_full(fd, write_buffer, left) != left) ? -1 : 0;
 }
 
 static void ce_smudge_racily_clean_entry(struct cache_entry *ce)
@@ -962,7 +1121,7 @@ static void ce_smudge_racily_clean_entry(struct cache_entry *ce)
 		 * size to zero here, then the object name recorded
 		 * in index is the 6-byte file but the cached stat information
 		 * becomes zero --- which would then match what we would
-		 * obtain from the filesystem next time we stat("frotz"). 
+		 * obtain from the filesystem next time we stat("frotz").
 		 *
 		 * However, the second update-index, before calling
 		 * this function, notices that the cached size is 6
@@ -974,11 +1133,13 @@ static void ce_smudge_racily_clean_entry(struct cache_entry *ce)
 	}
 }
 
-int write_cache(int newfd, struct cache_entry **cache, int entries)
+int write_index(struct index_state *istate, int newfd)
 {
 	SHA_CTX c;
 	struct cache_header hdr;
 	int i, removed;
+	struct cache_entry **cache = istate->cache;
+	int entries = istate->cache_nr;
 
 	for (i = removed = 0; i < entries; i++)
 		if (!cache[i]->ce_mode)
@@ -996,21 +1157,21 @@ int write_cache(int newfd, struct cache_entry **cache, int entries)
 		struct cache_entry *ce = cache[i];
 		if (!ce->ce_mode)
 			continue;
-		if (index_file_timestamp &&
-		    index_file_timestamp <= ntohl(ce->ce_mtime.sec))
+		if (istate->timestamp &&
+		    istate->timestamp <= ntohl(ce->ce_mtime.sec))
 			ce_smudge_racily_clean_entry(ce);
 		if (ce_write(&c, newfd, ce, ce_size(ce)) < 0)
 			return -1;
 	}
 
 	/* Write extension data here */
-	if (active_cache_tree) {
+	if (istate->cache_tree) {
 		unsigned long sz;
-		void *data = cache_tree_write(active_cache_tree, &sz);
+		void *data = cache_tree_write(istate->cache_tree, &sz);
 		if (data &&
 		    !write_index_ext_header(&c, newfd, CACHE_EXT_TREE, sz) &&
 		    !ce_write(&c, newfd, data, sz))
-			;
+			free(data);
 		else {
 			free(data);
 			return -1;
